﻿* Encoding: UTF-8.
*** Parallel Analysis.
*** The original test is described in: Horn, J. L. (1965). A Rationale and Test for the Number of Factors in Factor Analysis Psychometrika, 32, June, 179-185.
*** This syntax is taken from O'Connor (2000) SPSS and SAS Programs for Determining the Number of components Using Parallel Analysis and Velicer's MAP Test. 
*** Behavior Research Methods, Instruments, & Computers, 32 (3), 396-402.

set mxloops=9000 length=none printback=none width=80.
matrix.
*COMMENT: Enter the number of observations (Ncases) and number of variables (Nvars) here. The number of datasets (Ndatsets) and the percentile should remain as is.
compute Ncases = 921.
compute Nvars = 8.
compute Ndatsets = 1000.
compute percent = 95.
compute evals = make(nvars,ndatsets,-9999).
compute nm1 = 1 / (ncases-1).
loop #nds = 1 to ndatsets.
compute x = sqrt(2 * (ln(uniform(ncases,nvars)) * -1) ) &*
cos(6.283185 * uniform(ncases,nvars) ).
compute vcv = nm1 * (sscp(x) - ((t(csum(x))*csum(x))/ncases)).
compute d = inv(mdiag(sqrt(diag(vcv)))).
compute evals(:,#nds) = eval(d * vcv * d).
end loop.
compute num = rnd((percent*ndatsets)/100).
compute results = { t(1:nvars), t(1:nvars), t(1:nvars) }.
loop #root = 1 to nvars.
compute ranks = rnkorder(evals(#root,:)).
loop #col = 1 to ndatsets.
do if (ranks(1,#col) = num).
compute results(#root,3) = evals(#root,#col).
break.
end if.
end loop.
end loop.
compute results(:,2) = rsum(evals) / ndatsets.
compute specifs = {ncases; nvars; ndatsets; percent}.
print specifs /title="Specifications for this Run:"
/rlabels="Ncases" "Nvars" "Ndatsets" "Percent".
print results /title="Random Data Eigenvalues"
/clabels="Root" "Means" "Prcntyle".
end matrix.